class ImportS3UrlJob < Hyrax::ApplicationJob
  queue_as Hyrax.config.ingest_queue_name
  attr_reader :file_set, :bucket_name, :operation, :headers, :user, :source_object_key

  before_enqueue do |job|
    operation = job.arguments[1]
    operation.pending_job(job)
  end

  ##
  # @param [FileSet] file_set
  # @param [Hyrax::BatchCreateOperation] operation
  # @param [Hash] headers - header data to use in interaction with remote url
  # @param [Boolean] use_valkyrie - a switch on whether or not to use Valkyrie processing
  #
  # @todo At present, this job works for ActiveFedora objects. The use_valkyrie is not complete.
  def perform(file_set, operation, bucket_name, headers = {}, use_valkyrie: false)
    @file_set = file_set
    @operation = operation
    @headers = headers
    operation.performing!
    @user = User.find_by_user_key(file_set.depositor)
    @bucket_name = bucket_name
    @source_object_key = file_set.import_url

    if use_valkyrie
      # TODO
    else
      perform_af
    end
  end

  private

  def perform_af
    name = file_set.label

    # @todo Use Hydra::Works::AddExternalFileToFileSet instead of manually
    #       copying the file here. This will be gnarly.
    file_url = move_remote_file(name)
    file = handle_remote_file(file_url)
    # reload the FileSet once the data is movie since this is a long running task
    file_set.reload

    # FileSetActor operates synchronously so that this tempfile is available.
    # If asynchronous, the job might be invoked on a machine that did not have this temp file on its file system!
    # NOTE: The return status may be successful even if the content never attaches.
    log_import_status(file)
    file.close
  end

  def move_remote_file(name)
    parent_work = file_set.parent 
    s3 = S3StorageService.new
    s3.init_client
    target_object_key = file_set.import_url.split('/data/', 2)[1]
    file_set.import_url = target_object_key
    file_set.save!
    s3.copy_object(bucket_name, parent_work.id, source_object_key, target_object_key)
  rescue StandardError => e
    send_error(e.message)
  end

  ##
  # Send message to user on download failure
  #
  # @param error_message [String] the download error message
  def send_error(error_message)
    file_set.errors.add('Error:', error_message)
    Hyrax.config.callback.run(:after_import_url_failure, file_set, user, warn: false)
    operation.fail!(file_set.errors.full_messages.join(' '))
  end

  # Set the import operation status
  # @param f [IO] the stream to write to
  def log_import_status(f)
    if Hyrax::Actors::FileSetActor.new(file_set, user).create_content(f, from_url: true)
      operation.success!
    else
      send_error(source_object_key)
    end
  end

  def handle_remote_file(file_url)
    tmp_file = Tempfile.new()
    tmp_file.binmode

    tmp_file.write(file_url)

    tmp_file.rewind
    tmp_file
  end

end